#!/usr/bin/perl

# perfanalyze_dynamicsql.pl
#
# Last Updated: 2004/07/20 1123 CDT
# Description:
#    Analyzes dynamic sql snapshots for problem queries
# Usage:
#    This output takes a dynamic sql snapshot as input.
#    You can get a dynamic sql snapshot by running:
#       db2 get snapshot for dynamic sql on DBNAME
#    Alternatively (and perhaps better in general) is to turn
#    monitoring on at the database manager level:
#       db2 update database manager configuration using DFT_MON_STMT ON
#    Then restart your database:
#       db2stop
#       db2start

use Getopt::Std;
use POSIX;

getopts('i:d:o:t:c:hs');


# Print Usage and exit
if($opt_h) {
   print <<EOF

Usage: $0 [ -i inputFile | [ -d databaseName | -s ] ] [ -o outputFile ]
-i - file containing dynamic sql statements for processing
-d - database name to get dynamic sql statements from directly
-s - if given the temporary file with the dynamic sql will be saved
-o - file to put the processed results in, default is STDOUT
-t - length to truncate statement at, default is 90 characters. 0 = don't truncate.
-c - time cutoff; statements longer than this time are not included, default is 0.1

If no arguments are given, the program will read input from STDIN.

EOF
;
exit;
}

# Default input is STDIN
$fileHandle=*STDIN;

# Open an existing file if one is given
if(-f $opt_i) {
   open INPUT, $opt_i || die("Unable to open file $opt_i\n");
   $fileHandle=*INPUT;
}


# Fetch the data from the database ourselves
if($opt_d) {
   $date=strftime("%Y%m%d-%H%M",localtime);
   $tempFilename="snapshot-dynamicsql.$date";
   system("db2 get snapshot for dynamic sql on $opt_d > $tempFilename");
   open INPUT, $tempFilename || die("Unable to get dynamic sql from database, check directory permissions\n");
   $fileHandle=*INPUT;
}

if($opt_t ne "") {
   $statementTruncateLength=$opt_t;
} else {
   $statementTruncateLength=90;
}

if($opt_c ne "") {
   $timeCutoff=$opt_c;
} else {
   $timeCutoff=0.1;
}


# Default output file is STDOUT
$outputFileHandle=*STDOUT;

# Open the file to put the output in, if necessary
if($opt_o) {
   open OUTPUT, ">$opt_o" || die("Unable to open $opt_o for writing\n");
   $outputFileHandle=*OUTPUT;
}


# Now actually do the parsing
while($line=<$fileHandle>) {
   if($line=~/Number of executions\s+=\s+(.*)/i) {
      $numExecutions=$1;
   } elsif($line=~/Total execution time.*=\s+(.*)/i) {
      $totalExecutionTime=$1;
   } elsif($line=~/Statement text\s+=\s+(.*)/i) {
      $statement=$1;
   } elsif($line=~/Rows written\s+=\s+(.*)/i) {
      $rowsWritten=$1;
   }


   if($line=~/^$/ && $numExecutions>0) {
      $secPerExec=$totalExecutionTime/$numExecutions;
      $rowsPerExec=$rowsWritten/$numExecutions;

      $queryHash{$statement}{'statement'}=$statement;
      $queryHash{$statement}{'numExec'}=$numExecutions;
      $queryHash{$statement}{'totalExecTime'}=$totalExecTime;
      $queryHash{$statement}{'secPerExec'}=$secPerExec;
      $queryHash{$statement}{'rowsWritten'}=$rowsWritten;
      $queryHash{$statement}{'rowsPerExec'}=$rowsPerExec;
#
#     $statementNumExec{$statement}=$numExecutions;
#     $statementTotalExecTime{$statement}=$totalExecTime;
#     $statementSecPerExec{$statement}=$secPerExec;

      $sortHash{$statement}=$queryHash{$statement}{'secPerExec'};
      $statement="";
   }
}

#@sortedBySecPerExec = sort { $statementSecPerExec{$a} <=> $statementSecPerExec{$b} } keys %statementSecPerExec;
@sortedList = sort { $sortHash{$a} <=> $sortHash{$b} } keys %sortHash;


print $outputFileHandle "Legend\n";
print $outputFileHandle "   secPerExec - seconds per execution\n";
print $outputFileHandle "   numExec - number of executions\n";
print $outputFileHandle "   rowsW - number of rows written per execution\n";
print $outputFileHandle "   statement - statement executed\n";
print $outputFileHandle "Skipping statements that executed in less than $timeCutoff second...\n";
print $outputFileHandle "secPerExec  numExec   rowsW  statement\n";
foreach $statement (@sortedList) {
   $numExec=$queryHash{$statement}{'numExec'};
   $secPerExec=$queryHash{$statement}{'secPerExec'};
   $rowsPerExec=$queryHash{$statement}{'rowsPerExec'};

   next if($queryHash{$statement}{'secPerExec'}<=$timeCutoff);

   if($statementTruncateLength>0) {
      $statementTrunc=substr($statement,0,$statementTruncateLength) 
   } else {
      $statementTrunc=$statement;
   }

   printf $outputFileHandle "%10.4f  %5d  %8d  %s\n", $secPerExec, $numExec, $rowsPerExec, $statementTrunc;
}


# Clean up after ourselves
if($opt_d && !$opt_s) {
   unlink $tempFilename if(-f $tempFilename);
}
