#!/usr/bin/perl

# perfanalyze_tables.pl
#
# Last Updated: 2004/04/01 1326 CST
# Description:
#    Analyzes tables snapshots for problems 
# Usage:
#    This output takes a table snapshot as input.
#    You can get a tables snapshot by running:
#       db2 get snapshot for tables on DBNAME
#    Alternatively (and perhaps better in general) is to turn
#    monitoring on at the database manager level:
#       db2 update database manager configuration using DFT_MON_TABLE ON
#    Then restart your database:
#       db2stop
#       db2start

use Getopt::Std;
use POSIX;

getopts('i:d:o:c:hs');


# Print Usage and exit
if($opt_h) {
   print <<EOF

Usage: $0 [ -i inputFile | [ -d databaseName | -s ] ] [ -o outputFile ]
-i - file containing snapshot for processing
-d - database name to get snapshot from database directly
-s - if given the temporary file with the snapshot will be saved
-o - file to put the processed results in, default is STDOUT

If no arguments are given, the program will read input from STDIN.

EOF
;
exit;
}

# Default input is STDIN
$fileHandle=*STDIN;

# Open an existing file if one is given
if(-f $opt_i) {
   open INPUT, $opt_i || die("Unable to open file $opt_i\n");
   $fileHandle=*INPUT;
}


# Fetch the data from the database ourselves
if($opt_d) {
   $date=strftime("%Y%m%d-%H%M",localtime);
   $tempFilename="snapshot-tables.$date";
   system("db2 get snapshot for tables on $opt_d > $tempFilename");
   open INPUT, $tempFilename || die("Unable to get table snapshot from database, check directory permissions\n");
   $fileHandle=*INPUT;
}


# Default output file is STDOUT
$outputFileHandle=*STDOUT;

# Open the file to put the output in, if necessary
if($opt_o) {
   open OUTPUT, ">$opt_o" || die("Unable to open $opt_o for writing\n");
   $outputFileHandle=*OUTPUT;
}

if(!$opt_c) {
   $sortColumn="rowsRead";
} else {
   $sortColumn=$opt_c;
}


# Now actually do the parsing
while($line=<$fileHandle>) {
   if($line=~/Table Name\s+=\s+(.*)/i) {
      $tableName=$1;
      $tableHash{$tableName}{'tableName'}=$tableName;
   } elsif($line=~/Rows Read.*=\s+(.*)/i) {
      $tableHash{$tableName}{'rowsRead'}=$1;
   } elsif($line=~/Rows Written\s+=\s+(.*)/i) {
      $tableHash{$tableName}{'rowsWritten'}=$1;
   } elsif($line=~/Overflows\s+=\s+(.*)/i) {
      $tableHash{$tableName}{'overflows'}=$1;
   } elsif($line=~/Page Reorgs\s+=\s+(.*)/i) {
      $tableHash{$tableName}{'pageReorgs'}=$1;
   }

   $sortHash{$tableName}=$tableHash{$tableName}{$sortColumn};
}

@sortedList = sort { $sortHash{$a} <=> $sortHash{$b} } keys %sortHash;

print "tableName                rowsRead  rowsWritten    overflows   pageReorgs\n";
foreach $tableName (@sortedList) {
   $rowsRead=$tableHash{$tableName}{'rowsRead'};
   $rowsWritten=$tableHash{$tableName}{'rowsWritten'};
   $overflows=$tableHash{$tableName}{'overflows'};
   $pageReorgs=$tableHash{$tableName}{'pageReorgs'};
   printf "%-20s   %10d   %10d   %10d   %10d\n", $tableName, $rowsRead, $rowsWritten, $overflows, $pageReorgs;
}

# Clean up after ourselves
if($opt_d && !$opt_s) {
   unlink $tempFilename if(-f $tempFilename);
}
